--[[
	Kui Mount
	Kesava-Auchindoun
]]
local addon,ns = ...
local category = 'Kui Mount'
do
	--------------------------------------- Create interface options category --
	local opt = CreateFrame("Frame", "KuiMountConfig", InterfaceOptionsFramePanelContainer)
	opt:Hide()
	opt.name = category

	local blacklistTitle = opt:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	blacklistTitle:SetText('Blacklist')
	blacklistTitle:SetPoint('TOPLEFT', 16, -16)

	local blacklistDesc = opt:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
	blacklistDesc:SetText('Type the names of mounts you want Kui Mount to never use.')
	blacklistDesc:SetPoint('LEFT', blacklistTitle, 'RIGHT', 5, 0)
	
	local blacklistInput = CreateFrame('EditBox', 'KuiMountBlacklistInput', opt, 'InputBoxTemplate')
	blacklistInput:SetPoint('TOPLEFT', blacklistTitle, 'BOTTOMLEFT', 0, -16)
	blacklistInput:SetAutoFocus(false)
	blacklistInput:EnableMouse(true)
	blacklistInput:SetWidth(300)
	blacklistInput:SetHeight(20)
	blacklistInput:Show()
	
	local blacklistInputButton = CreateFrame('Button', KuiMountBlacklistInputButton, blacklistInput, 'UIPanelButtonTemplate')
	blacklistInputButton:SetWidth(300)
	blacklistInputButton:SetHeight(20)
	blacklistInputButton:SetPoint('LEFT', blacklistInput, 'RIGHT')
	blacklistInputButton:SetText('Add/Remove')
	
	local blacklistBox = CreateFrame("EditBox", 'KuiMountBlacklistBox', opt)
	blacklistBox:SetMultiLine(true)
	blacklistBox:SetAutoFocus(false)
	blacklistBox:EnableMouse(false)
	blacklistBox:SetFontObject(ChatFontNormal)
	blacklistBox:SetWidth(570)
	blacklistBox:SetHeight(450)
	blacklistBox:Show()
	
	local blacklistScroll = CreateFrame('ScrollFrame', 'KuiMountBlacklistScroll', opt, 'UIPanelScrollFrameTemplate')
	blacklistScroll:SetPoint('TOPLEFT', blacklistInput, 'BOTTOMLEFT', 0, -16)
	blacklistScroll:SetWidth(570)
	blacklistScroll:SetHeight(450)
	blacklistScroll:SetScrollChild(blacklistBox)

	local blacklistBackdrop = CreateFrame('Frame', nil, opt)
	blacklistBackdrop:SetBackdrop({
		bgFile = 'Interface\\ChatFrame\\ChatFrameBackground',
		edgeFile = 'Interface\\Tooltips\\UI-Tooltip-border',
		edgeSize = 16,
		insets = { left = 4, right = 4, top = 4, bottom = 4 }
	})
	blacklistBackdrop:SetBackdropColor(.1, .1 , .1, .3)
	blacklistBackdrop:SetBackdropBorderColor(.5, .5, .5)
	blacklistBackdrop:SetPoint('TOPLEFT', blacklistScroll, -10, 10)
	blacklistBackdrop:SetPoint('BOTTOMRIGHT', blacklistScroll, 30, -10)
	
	----------------------------------------------------------------- Scripts --
	local function OnOptionsShow(self)
		local text, name, _
		for name, _ in pairs(KuiMountSaved.blacklist) do
			text = (text and text..'\n'..name or name)
		end
		
		blacklistBox:SetText(text or '')
	end

	local nummounts, mountlist = 0, {}
	local function OnBlacklistButtonClick(self)
		if GetNumCompanions('mount') ~= nummounts then
			-- repopulate the mount list
			nummounts = GetNumCompanions('mount')
			mountlist = {}
			
			local i
			for i = 1, nummounts do
				local _, mountname = GetCompanionInfo('mount', i) 
				mountlist[strlower(mountname)] = mountname
			end
		end

		local name = blacklistInput:GetText()
		local rname = mountlist[strlower(name)]
		
		if rname then
			if not KuiMountSaved.blacklist[rname] then
				KuiMountSaved.blacklist[rname] = true
			else
				KuiMountSaved.blacklist[rname] = nil
			end
		
			blacklistInput:SetText('')
		else
			if KuiMountSaved.blacklist[name] then
				-- remove a mount which no longer exists
				KuiMountSaved.blacklist[name] = nil
			else			
				-- show error
				print('Kui_Mount: |cffff0000`'..name..'` isn\'t the name of a mount in your spellbook. Did you type it wrong?')
			end
		end
		
		OnOptionsShow(KuiMountConfig)
	end
	
	opt:SetScript('OnShow', OnOptionsShow)
	
	blacklistInputButton:SetScript('OnClick', OnBlacklistButtonClick)
	
	blacklistInput:SetScript('OnEnterPressed', function(self)
		blacklistInputButton:Click()
	end)
	
	blacklistInput:SetScript('OnEscapePressed', function(self)
		self:SetText('')
		self:ClearFocus()
	end)
	
	InterfaceOptions_AddCategory(opt)
end

--------------------------------------------------------------- Slash command --
SLASH_KUIMOUNT1 = '/kuimount'
SLASH_KUIMOUNT2 = '/mount'

function SlashCmdList.KUIMOUNT(msg)
	if msg ~= '' then
		InterfaceOptionsFrame_OpenToCategory(category)
	else
		ns.Mount()
	end
end